<?php

namespace Drupal\universal_entity_api\Normalizer;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\universal_entity_api\Normalizer\ParagraphNormalizer;
use Drupal\universal_entity_api\Normalizer\MediaNormalizer;
use Drupal\Core\Entity\EntityInterface;

class FieldNormalizer implements ContainerInjectionInterface {

  protected $paragraphNormalizer;
  protected $mediaNormalizer;
  protected $fileUrlGenerator;

  public function __construct(
    ParagraphNormalizer $paragraphNormalizer,
    MediaNormalizer $mediaNormalizer,
    FileUrlGeneratorInterface $fileUrlGenerator
  ) {
    $this->paragraphNormalizer = $paragraphNormalizer;
    $this->mediaNormalizer = $mediaNormalizer;
    $this->fileUrlGenerator = $fileUrlGenerator;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('universal_entity_api.paragraph_normalizer'),
      $container->get('universal_entity_api.media_normalizer'),
      $container->get('file_url_generator')
    );
  }

  public function normalize(FieldItemListInterface $field) {
    $type = $field->getFieldDefinition()->getType();
    $entities = method_exists($field, 'referencedEntities') ? $field->referencedEntities() : [];

    // Paragraphs
    if ($type === 'entity_reference_revisions' && !empty($entities)) {
      if ($entities[0]->getEntityTypeId() === 'paragraph') {
        return $this->paragraphNormalizer->normalize($entities);
      }
    }

    // Media
    if ($type === 'entity_reference' && !empty($entities)) {
      if ($entities[0]->getEntityTypeId() === 'media') {
        return array_map(fn($media) => $this->mediaNormalizer->normalize($media), $entities);
      }

      // Fallback for other entity types — minimal data
      return array_map(function (EntityInterface $e) {
        return [
          'id' => $e->id(),
          'type' => $e->getEntityTypeId(),
          'bundle' => $e->bundle(),
          'label' => $e->label(),
        ];
      }, $entities);
    }

    // Image fields
    if ($type === 'image') {
      return array_map(function ($item) {
        if (!isset($item->entity)) {
          return [];
        }
        return [
          'url' => $this->fileUrlGenerator->generateAbsoluteString($item->entity->getFileUri()),
          'alt' => $item->alt ?? '',
          'title' => $item->title ?? '',
          'width' => $item->width ?? null,
          'height' => $item->height ?? null,
        ];
      }, iterator_to_array($field));
    }

    // Taxonomy term references
    if (!empty($entities) && $entities[0]->getEntityTypeId() === 'taxonomy_term') {
      return array_map(function ($term) {
        return [
          'tid' => $term->id(),
          'name' => $term->label(),
          'description' => $term->getDescription(),
          'vocabulary' => $term->bundle(),
        ];
      }, $entities);
    }

    // Default
    return $field->getValue();
  }
}
